/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp remote_controller.xml -a remotecontroller
 *
 * qdbusxml2cpp is Copyright (C) 2017 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef REMOTECONTROLLER_H
#define REMOTECONTROLLER_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
QT_BEGIN_NAMESPACE
class QByteArray;
template<class T> class QList;
template<class Key, class Value> class QMap;
class QString;
class QStringList;
class QVariant;
QT_END_NAMESPACE

/*
 * Adaptor class for interface com.kylin.kylinvideo.controller
 */
class ControllerAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "com.kylin.kylinvideo.controller")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"com.kylin.kylinvideo.controller\">\n"
"    <method name=\"seek_forward\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"      <arg direction=\"in\" type=\"i\" name=\"seconds\"/>\n"
"    </method>\n"
"    <method name=\"seek_rewind\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"      <arg direction=\"in\" type=\"i\" name=\"seconds\"/>\n"
"    </method>\n"
"    <method name=\"playPause\"/>\n"
"    <method name=\"stop\"/>\n"
"    <method name=\"quit\"/>\n"
"    <signal name=\"aboutToQuit\"/>\n"
"  </interface>\n"
        "")
public:
    ControllerAdaptor(QObject *parent);
    virtual ~ControllerAdaptor();

public: // PROPERTIES
public Q_SLOTS: // METHODS
    void playPause();
    void quit();
    bool seek_forward(int seconds);
    bool seek_rewind(int seconds);
    void stop();
Q_SIGNALS: // SIGNALS
    void aboutToQuit();
};

#endif
